/*
Copyright (c) 2008-2010 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Description: 
*/


#ifndef EXAMPLE_H
#define EXAMPLE_H

#define KMaxEntryCount 5    //Keeping message entry count.
class CSearchsortExample;    //Forward Declaration.

/**
Reads keypresses in asynchronous way
and calls example functions.
*/

class CKeyReader : public CActive
	{

public:

	/**
	Constructor.
	*/
	CKeyReader(CConsoleBase* aConsole,CSearchsortExample* aExample);

	/**
	Desructor.
	*/
	~CKeyReader();

	/**
	Request Function, this takes the key inputs.
	*/
	void StartL();

	void RunL();
	void DoCancel();

	/**
	Handling leaving of RunL.
	*/
	TInt RunError(TInt aError);


	/**
	Function for making search-sort settings.
	*/
	TBool SearchSortSettings();

	/**
	Function for resetting search-sort settings.
	*/
	void ResetSettings();

	/**
	Function for creating message entries.
	*/
	void CreateMessageEntriesL();

	/**
	Function for displaying message summary.
	*/
	void DisplayMessageSummary();
public:

	/**
	Buffer for getting message header info
	*/
	TBuf<32> iFrominfo[KMaxEntryCount];
	TBuf<32> iToinfo[KMaxEntryCount];
	TBuf<32> iSubjectinfo[KMaxEntryCount];

	/**
	Variables taking user input and hence used in search-sort settings
	*/
	TBuf<64> iSender;
	TBool iCaseSensitivityON,iWholeWordSearch,iCombinedSearch;
private:
	/**
	Pointer to console.
	Not owned.
	*/
	CConsoleBase* iConsole;

	/**
	Pointer to CSearchsortExample.
	*/
	CSearchsortExample* iSearchSortExample;

	/**
	Leave code catch.
	*/
	TInt iErrorCondition;
	/**
	search-sort Result count.
	*/
	TInt iResultCount;

	};

#endif//EXAMPLE_H
